@@ -20,7 +20,7 @@ def call_process(args, cwd='.', input=None, verbose=False, print_on_error=True,
     try:
         # check_output will not print
         # note that we redirect stderr to stdout!
-        subprocess.run(args=args, stdout=print_utils.log_file, stderr=subprocess.STDOUT, cwd=cwd, input=input, shell=shell)
+        subprocess.check_output(args=args, stdout=print_utils.log_file, stderr=subprocess.STDOUT, cwd=cwd, input=input, shell=shell)
     except subprocess.CalledProcessError as e:
         # on error, print output (includes stderr)
         if print_on_error:
